#############################################################
#
# kernel
#
#############################################################

KERNEL_VER:=2.6.20.4
KERNEL_DIR:=$(BUILD_DIR)/linux-$(KERNEL_VER)
KERNEL_SOURCE:=linux-$(KERNEL_VER).tar.bz2
KERNEL_SITE:=http://www.kernel.org/pub/linux/kernel/v2.6
KERNEL_UNZIP=bzcat
KERNEL_VMLINUZ:=mb-vmlinuz-$(KERNEL_VER)
KERNEL_MODULES:=kmodules-$(KERNEL_VER).tar.bz2


KERNEL_CONFIG_FILE=$(subst ",, $(strip $(BR2_PACKAGE_KERNEL_CONFIG)))
#"

$(DL_DIR)/$(KERNEL_SOURCE):
	 $(WGET) -P $(DL_DIR) $(KERNEL_SITE)/$(KERNEL_SOURCE)

kernel-source: $(DL_DIR)/$(KERNEL_SOURCE) $(KERNEL_CONFIG_FILE)

$(KERNEL_DIR)/.unpacked: $(DL_DIR)/$(KERNEL_SOURCE)
	$(KERNEL_UNZIP) $(DL_DIR)/$(KERNEL_SOURCE) | tar -C $(BUILD_DIR) $(TAR_OPTIONS) -
	# Allow kernel patches.
	toolchain/patch-kernel.sh $(KERNEL_DIR) package/kernel kernel\*.patch
	touch $(KERNEL_DIR)/.unpacked

$(KERNEL_DIR)/.configured: $(KERNEL_DIR)/.unpacked $(KERNEL_CONFIG_FILE)
	cp $(KERNEL_CONFIG_FILE) $(KERNEL_DIR)/.config
	yes "" | $(MAKE) -C $(KERNEL_DIR) oldconfig
	touch $(KERNEL_DIR)/.configured

$(KERNEL_DIR)/arch/i386/boot/bzImage: $(KERNEL_DIR)/.configured
	$(MAKE) -C $(KERNEL_DIR)

$(BIN_DIR)/$(KERNEL_VMLINUZ): $(KERNEL_DIR)/arch/i386/boot/bzImage
	cp $(KERNEL_DIR)/arch/i386/boot/bzImage $(BIN_DIR)/$(KERNEL_VMLINUZ)
	touch -c $(BIN_DIR)/$(KERNEL_VMLINUZ)

$(BIN_DIR)/$(KERNEL_MODULES): $(KERNEL_DIR)/arch/i386/boot/bzImage
	-rm -rf $(PACK_DIR)/kmodules
	-mkdir $(PACK_DIR)/kmodules
	make INSTALL_MOD_PATH=$(PACK_DIR)/kmodules -C $(KERNEL_DIR) modules_install

	tar -cjf $(BIN_DIR)/$(KERNEL_MODULES) -C $(PACK_DIR)/kmodules .
	touch -c $(BIN_DIR)/$(KERNEL_MODULES)

kernel-prepare:
	touch $(KERNEL_DIR)/.configured

kernel-config: $(KERNEL_DIR)/.configured
	$(MAKE) -C $(KERNEL_DIR) menuconfig

kernel-save-config: $(KERNEL_DIR)/.config
	cp $(KERNEL_DIR)/.config $(KERNEL_CONFIG_FILE)

kernel: $(BIN_DIR)/$(KERNEL_VMLINUZ) $(BIN_DIR)/$(KERNEL_MODULES)

kernel-clean:
	-$(MAKE) -C $(KERNEL_DIR) clean

kernel-dirclean:
	rm -rf $(KERNEL_DIR)
#############################################################
#
# Toplevel Makefile options
#
#############################################################
ifeq ($(strip $(BR2_PACKAGE_KERNEL)),y)
BINDIR_TARGETS+=kernel
endif
