import os.path

# Specify the kernel version which used by MI environment.
mikernelver = '2.6.28.9-smp'

# If .distmode exists, enable Distributor Mode
develmode = not os.path.exists('.distmode')

#{{{ User custom default value
welcome = 'Welcome to the Auto Distribution provided by MagicInstaller.'
distname = 'AutoDistribution'
distver = '0.0'
distkernelver = '2.6.9'
pkgtype = 'rpm'
langset = 'en:zh_CN'

if mikernelver[:3]=='2.4':
    kmods_arrange = {
        "boot" : [ '@ext3.o', '@jfs.o', '@ntfs.o', '@reiserfs.o', '@vfat.o' ],
        "scsi" : [ '@xfs.o', '@sd_mod.o', '@kernel/drivers/md', '@kernel/drivers/scsi' ],
        "net" : ['@xfs.o', '@kernel/drivers/md', '@kernel/drivers/net' ]
        }
else:
    kmods_arrange = {
        "boot" : [ '@jfs.ko', '@ntfs.ko', '@vfat.ko' ],
        "scsi" : [ '@xfs.ko', '@sd_mod.ko', '@kernel/drivers/scsi' ],
        "net" : ['@xfs.ko', '@kernel/drivers/net' ]
        }

autopart_profile = {}

specdir = 'spec'
specfn  = 'spec/specinfo.py'
pkgdirs = specdir + '/packages'
hotfixfiles_dir = 'spec/hotfix'
addfiles_dir = 'spec/addfiles'

dolog = True                           # Log message or not
nonewt = False                          # Without newt or not
useudev = True                         # Use udev or not
bootload = 'grub'

volume_limit_list = [620 * 1024 * 1024, 640 * 1024 * 1024, 640 * 1024 * 1024]
placement_list = [[]]
toplevel_groups = {}
add_deps = {}
remove_deps = {}

# merge specinfo
import specinfo
for key in dir(specinfo):
    if (key[:2] != '__'):
        vars()[key] = specinfo.__dict__[key]
del specinfo

# iso name
max_cd_no = 9
isofn_fmt = '%s-%s-%d.iso'
def mkisofn(iso_no):
    global distname, distver
    return 'result/' + isofn_fmt % (distname, distver, iso_no)
bootiso_fn = os.path.basename(mkisofn(1))

if useudev:
    udev_arg = '--use-udev'
else:
    udev_arg = ''
# }}}

# {{{ Run setting
# boot screen mode
gui_mode =  785 #788

if gui_mode == 785:
    full_width  = 640
    full_height = 480
elif gui_mode == 788:
    full_width  = 800
    full_height = 600
gui_resolution = '%dx%d' % (full_width, full_height)

# size (K bytes) of memory driver holding MI image.
ramdisk_size = 20 * 1024                #19

hotfixdir = '/tmp/update'
# }}}

# {{{ Build environment
bindir = os.path.abspath('bindir')
srcdir = os.path.abspath('srcdir')
tmpdir = os.path.abspath('tmp')

# in tmpdir, for building only
destdir = os.path.join(tmpdir, 'root.src')     # with magicinstaller root src files
mbdestdir = os.path.join(tmpdir, 'mbroot.src') # with magicbooter root src files
devrootdir = os.path.join(tmpdir, 'devroot')   # building sources dir
mbdevrootdir = os.path.join(tmpdir, 'mbdevroot')   # building mb sources dir
bootcd_dir = os.path.join(tmpdir, 'bootcd')    # bootcd dir
miimages_cddir = 'boot'
miimages_dir = os.path.join(bootcd_dir, miimages_cddir) # MI images dir in bootcd
mlbase_dir = os.path.join(bootcd_dir, distname, 'base') # dist base dir in bootcd

root_bindir = '/usr/bin'
root_datadir = '/usr/share/MagicInstaller'
root_libdir = '/$pythondir/site-packages'
root_moduledir = '/usr/share/MagicInstaller/modules'
root_operationdir = '/usr/share/MagicInstaller/operations'

# Python
pythondir = 'usr/lib/python2.4'
pythonbin = '/usr/bin/python24'

# cross compile (uclibc)
cross_compile = '/opt/MI-linux-uclibc/bin/i686-linux-uclibc-'

# Busybox
busybox_version = '1.6.1'

# Udev
udev_version = '100'

# i18n translation
textdomain = 'magic.installer'
translators = ''
copyright_holder = 'Charles Wang'
all_linguas = ['zh_CN']

lang_map = { 'ja_JP': ('eucJP', 'eucJP'),
             'ko_KR' : ('eucKR', 'eucKR'),
             'zh_CN': ('gb2312', 'GB2312'),
             'zh_TW': ('big5', 'BIG5') }
#}}}
