#!/usr/bin/python
# Copyright (C) 2003, Charles Wang.
# Author:  Charles Wang <charles@linux.net.cn>
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANT; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public LIcense for more
# details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, 59 Temple
# Place - Suite 330, Boston, MA 02111-1307, USA.


import os
import sys
import types

from getopt import getopt

import pkgpublic

# Usage:
#   PkgArrange.py [-o outputfn] [-l logfn]

try:
    from pkginfor import packages_infor
except ImportError:
    sys.path.insert(0, 'tmp')
    from pkginfor import packages_infor

sys.path.insert(0, '')
sys.path.insert(0, 'spec')
import mi_config

volume_limit_list   = mi_config.volume_limit_list
placement_list      = mi_config.placement_list
toplevel_groups     = mi_config.toplevel_groups
add_deps            = mi_config.add_deps
remove_deps         = mi_config.remove_deps

# To resolve the loop dependency, there are two method.
# The first is use remove_deps to broke the loop. This method can control the
#  consequence of the package but lost dependency link. So group make must
#  remember to include them both or do not include any one.
# The second method is leave the loop dependency for this script. The
#  dependency link is reserved, but the packages consequence is random...


optlist, args = getopt(sys.argv[1:], 'o:l:')

outfd = 1
logfd = 2
for (optname, optval) in optlist:
    if optname == '-o':
        if outfd > 2:
            os.close(outfd)
        try:
            outfd = os.open(optval, os.O_WRONLY | os.O_CREAT | os.O_TRUNC)
        except OSError:
            outfd = 1
    elif optname == '-l':
        if logfd > 2:
            os.close(logfd)
        try:
            logfd = os.open(optval, os.O_WRONLY | os.O_CREAT | os.O_TRUNC, 0644)
        except OSError:
            logfd = 2


# Adjust dependencies according to add_deps, remove_deps
adjdeps_map = {}
for k in add_deps.keys():
    adjdeps_map[k] = 'y'
for k in remove_deps.keys():
    adjdeps_map[k] = 'y'
for pkg in adjdeps_map.keys():
    add_map = {}
    remove_map = {}
    if add_deps.has_key(pkg):
        for dep in add_deps[pkg]:
            add_map[dep] = 'y'
    if remove_deps.has_key(pkg):
        for dep in remove_deps[pkg]:
            remove_map[dep] = 'y'
    for dep in packages_infor[pkg][pkgpublic.deps]:
        if not remove_map.has_key(dep):
            add_map[dep] = 'y'
    packages_infor[pkg][pkgpublic.deps] = add_map.keys()


# Remove the bypass dependencies in packages_infor.
for pkgfn in packages_infor.keys():
    topdeps = packages_infor[pkgfn][pkgpublic.deps]
    curdeplist = []
    for dep in topdeps:
        curdeplist.append((dep, []))
    subdeps_map = {}
    loopdeps_map = {}
    while curdeplist != []:
        newdeplist = []
        for (dep, deppath) in curdeplist:
            newdeppath = deppath + [dep]
            if dep == pkgfn:
                os.write(logfd, 'LoopDep: %s\n' % str(newdeppath))
                for d in deppath:
                    loopdeps_map[d] = 'y'
                continue # Do not go more on loop dependency.
            depdeplist = packages_infor[dep][pkgpublic.deps]
            for depdep in depdeplist:
                if subdeps_map.has_key(depdep):
                    continue
                if deppath != [] and depdep in topdeps:
                    #print pkgfn, newdeppath, depdep
                    continue
                newdeplist.append((depdep, newdeppath))
                subdeps_map[depdep] = str(newdeppath)
        curdeplist = newdeplist
    newtopdeps = []
    # Loop Dependency should be reserved.
    for dep in loopdeps_map.keys():
        if dep != pkgfn:
            newtopdeps.append(dep)
    for dep in topdeps:
        if loopdeps_map.has_key(dep):
            continue
        if subdeps_map.has_key(dep):
            os.write(logfd, '%s is removed from the deplist of %s because %s.\n' % (dep, pkgfn, subdeps_map[dep]))
            continue
        newtopdeps.append(dep)
    packages_infor[pkgfn][pkgpublic.deps] = newtopdeps
    del(subdeps_map) # Release the space occupied by subdeps_map.

# Sort according dependency.
minset_list = []
minset_map = {}

def recur_deps(pkglist, curmap):
    # pkglist might be duplicated but curmap can skip it.
    if pkglist == []:
        return []
    result = []
    for pkg in pkglist:
        if minset_map.has_key(pkg):
            continue
        if curmap.has_key(pkg):
            continue
        curmap[pkg] = 'y'
        pkgdeplist = packages_infor[pkg][pkgpublic.deps]
        result = result + recur_deps(pkgdeplist, curmap) + [pkg]
    return result

for placement in placement_list:
    result_list = recur_deps(placement, {})
    minset_list.append(result_list)
    for result in result_list:
        minset_map[result] = 'y'

# Search all packages that are not arrange yet.
last_list = []
for pkgfn in packages_infor.keys():
    if minset_map.has_key(pkgfn):
        continue
    last_list.append(pkgfn)

result_list = recur_deps(last_list, {})
minset_list.append(result_list)

del(minset_map) # Free the space.

# Now the minset_list contain the minimum set for all placement set.
volume_size = 0
curlist = []
arrangement_list = []
minset_count = 0
volume_count = 0
pkg_count = 0
pkgpos_map = {}

for minset in minset_list:
    for pkgfn in minset:
        pkgsize = packages_infor[pkgfn][pkgpublic.totalsz]
        if volume_size + pkgsize > volume_limit_list[0]:
            if minset_count <= volume_count:
                errmsg = 'Not enough space to place the package "%s" in disc #%d.\n' % (pkgfn, (minset_count + 1))
                os.write(logfd, errmsg)
                if logfd != 2:
                    os.write(2, errmsg)
                sys.exit(1)
            arrangement_list.append(curlist)
            curlist = []
            volume_size = 0
            del volume_limit_list[0]
            volume_count = volume_count + 1
            if volume_limit_list == []:
                errmsg = 'Not enough volume! Broken on %s.\n' % pkgfn
                os.write(logfd, errmsg)
                if logfd != 2:
                    os.write(2, errmsg)
                sys.exit(1)
            pkg_count = 0
        volume_size = volume_size + pkgsize
        curlist.append(packages_infor[pkgfn])
        pkgpos_map[pkgfn] = (volume_count, pkg_count)
        pkg_count = pkg_count + 1
    minset_count = minset_count + 1

if curlist != []:
    arrangement_list.append(curlist)

# Calculate total package size for all arch.
archsize_map = {}
for arch in pkgpublic.arch_map.keys():
    archsize_map[arch] = 0
for arrangement in arrangement_list:
    for pkginfor in arrangement:
        size_map = {}
        noarchsz = -1
        for pkgarch in pkginfor[pkgpublic.pathes]:
            if pkgarch[1] == 'noarch':
                noarchsz = pkgarch[2]
            else:
                size_map[pkgarch[1]] = pkgarch[2]
        if noarchsz >= 0:
            for arch in archsize_map.keys():
                archsize_map[arch] = archsize_map[arch] + noarchsz
        else:
            for arch in archsize_map.keys():
                if size_map.has_key(arch):
                    archsize_map[arch] = archsize_map[arch] + size_map[arch]
                    continue
                for compat in pkgpublic.arch_map[arch]:
                    if size_map.has_key(compat):
                        archsize_map[arch] = archsize_map[arch] + size_map[compat]
                        break

# Output 'arrangement' and 'pkgpos_map' for MagicInstaller.
os.write(outfd, '#!/usr/bin/python\n')

result = 'arch_map = {'
allkeys = pkgpublic.arch_map.keys()
allkeys.sort()
sep = '\n\t'
for arch in allkeys:
    result = result + sep + "'%s' : %s" % (arch, pkgpublic.arch_map[arch])
    sep = ',\n\t'
result = result + ' }\n'
os.write(outfd, result + '\n')

result = 'arrangement = ['
sep = ''
for arrangement in arrangement_list:
    result = result + sep + "\n["
    sep1 = ''
    for pkginfor in arrangement:
        result = result + sep1 + \
                 "\n\t[%dL, '%s',\n" % (pkginfor[pkgpublic.totalsz],
                                        os.path.basename(pkginfor[pkgpublic.path]))
        result = result + "\t\t%s," % str(pkginfor[pkgpublic.group])
        deplist = pkginfor[pkgpublic.deps]
        if deplist == []:
            result = result + ' [], '
        else:
            for i in range(len(deplist)):
                if i == 0:
                    result = result + '\n\t\t['
                else:
                    result = result + '\t\t '
                dep = deplist[i]
                if type(dep) == types.StringType:
                    result = result + "'%s'" % dep
                else:
                    result = result + '%s' % str(dep)
                if i != len(deplist) - 1:
                    result = result + ',\n'
                else:
                    result = result + '],'
        pathes = pkginfor[pkgpublic.pathes]
        if pathes == []:
            result = result + ' []]'
        else:
            for i in range(len(pathes)):
                if i == 0:
                    result = result + '\n\t\t['
                else:
                    result = result + '\t\t '
                result = result + '%s' % str(pathes[i])
                if i != len(pathes) - 1:
                    result = result + ',\n'
                else:
                    result = result + ']]'
        sep1 = ','
    result = result + "]\n"
    sep = ','
result = result + "]\n"
os.write(outfd, result + '\n')

result = 'archsize_map = {'
allkeys = archsize_map.keys()
allkeys.sort()
sep = '\n\t'
for arch in allkeys:
    result = result + sep + "'%s' : %s" % (arch, archsize_map[arch])
    sep = ',\n\t'
result = result + ' }\n'
os.write(outfd, result + '\n')

result = 'pkgpos_map = {'
sep = ''
for pkg in pkgpos_map.keys():
    result = result + sep + "\n\t'%s': %s" % (pkg, str(pkgpos_map[pkg]))
    sep = ','
result = result + '}\n'
os.write(outfd, result + '\n')

toplevelgrp_map = {}
for group in toplevel_groups.keys():
    toplevelgrp_map[group] = {}
    for pkg in toplevel_groups[group]:
        if not pkgpos_map.has_key(pkg):
            # Skip the invalid package.
            continue
        pkglist = [pkg]        
        while pkglist != []:
            newpkglist = []
            for pkg in pkglist:
                if toplevelgrp_map[group].has_key(pkg):
                    continue
                toplevelgrp_map[group][pkg] = 'y'
                (volno, pkgno) = pkgpos_map[pkg]
                pkgdeps = arrangement_list[volno][pkgno][3]
                for pkgdep in pkgdeps:
                    if not toplevelgrp_map[group].has_key(pkgdep):
                        newpkglist.append(pkgdep)
            pkglist = newpkglist

result = 'toplevelgrp_map = {'
sep = ''
for group in toplevelgrp_map.keys():
    result = result + sep + "\n\t'%s': {" % group
    sep1 = ''
    for pkg in toplevelgrp_map[group].keys():
        result = result + sep1 + "\n\t\t'%s': '%s'" % (pkg, group)
        sep1 = ','
    result = result + '}'
    sep = ','
result = result + '}\n'
os.write(outfd, result + '\n')
